from pymclevel import TAG_Byte, TAG_Short, TAG_Int, TAG_Compound, TAG_List, TAG_String, TAG_Double, TAG_Float, TAG_Long
from copy import deepcopy

displayName = "Chest Contents to Command Block"
inputs = (
	("Command Block Type:",("Command Blocks","Minecart Command Blocks")),
	("Command Type:",("Summon","Give")),
	("Target (Give Commands Only):",("string","value=@a")),
	("Coordinates are Relative to Command Block",True),
	("X Spawn Position",0.0),
	("Y Spawn Position",0.0),
	("Z Spawn Position",0.0),
	)

def Spawner2Command(entity):
	command = ""
	if type(entity) is TAG_List:
		list = True
	else:
		list = False

	for tag in range(0,len(entity)) if list else entity.keys():
		if type(entity[tag]) is TAG_Compound:
			if not list:
				if tag != "":
					command += tag+":"
			command += "{"
			command += Spawner2Command(entity[tag])
			command += "}"
		elif type(entity[tag]) is TAG_List:
			if not list:
				if tag != "":
					command += tag+":"
			command += "["
			command += Spawner2Command(entity[tag])
			command += "]"
		else:
			if type(entity[tag]) is TAG_String:
				if entity[tag].value == "":
					continue
			if not list:
				if tag != "":
					command += tag+":"
			if type(entity[tag]) is TAG_String:
				command += "\""
				command += str.replace(entity[tag].value.encode("unicode-escape"), r'"',r'\\"')
				command += "\""
			else:
				command += entity[tag].value.encode("unicode-escape") if isinstance(entity[tag].value, unicode) else str(entity[tag].value)
				if type(entity[tag]) is TAG_Byte:
					command += "b"
				elif type(entity[tag]) is TAG_Short:
					command += "s"
				elif type(entity[tag]) is TAG_Long:
					command += "l"
				elif type(entity[tag]) is TAG_Float:
					command += "f"
				elif type(entity[tag]) is TAG_Double:
					command += "d"
			
		command += ","
	else:
		if command != "":
			if command[-1] == ",":
				command = command[:-1]
	return command


def perform(level, box, options):
	blocktype = options["Command Block Type:"]
	command = options["Command Type:"]
	target = options["Target (Give Commands Only):"]
	relative = options["Coordinates are Relative to Command Block"]
	posx = options["X Spawn Position"]
	posy = options["Y Spawn Position"]
	posz = options["Z Spawn Position"]
	entsToAdd = []
	entsToDelete = []
	for (chunk, _, _) in level.getChunkSlices(box):
		for e in chunk.TileEntities:
			x = e["x"].value
			y = e["y"].value
			z = e["z"].value
			if (x,y,z) in box:
				if e["id"].value != "Chest":
					continue
				if "Items" not in e:
					continue
				if len(e["Items"]) <= 0:
					continue

				ent = TAG_Compound()
				if len(e["Items"]) == 1:
					if e["Items"][0]["id"].value == 401:
						type = "FireworksRocketEntity"
						ent["id"] = TAG_String("FireworksRocketEntity")
						ent["Life"] = TAG_Int(0)
						ent["LifeTime"] = TAG_Int(28)
						ent["FireworksItem"] = deepcopy(e["Items"][0])
						if "Slot" in ent["FireworksItem"]:
							del ent["FireworksItem"]["Slot"]
					else:
						ent["id"] = TAG_String("Item")
						type = "Item"
						ent["Item"] = deepcopy(e["Items"][0])
						if "Slot" in ent["Item"]:
							del ent["Item"]["Slot"]
				else:
					ent["id"] = TAG_String("Item")
					type = "Item"
					entptr = ent
					for item in e["Items"]:
						entptr["Item"] = deepcopy(item)
						entptr["id"] = TAG_String("Item")
						if "Slot" in entptr["Item"]:
							del entptr["Item"]["Slot"]
						entptr["Riding"] = TAG_Compound()
						entptr = entptr["Riding"]

						
				spawndata = "{"+Spawner2Command(ent)+"}"
				if relative:
					sx = "~" + (str(posx) if posx != 0.0 else "")
					sy = "~" + (str(posy) if posy != 0.0 else "")
					sz = "~" + (str(posz) if posz != 0.0 else "")
				else:
					sx = str(posx)
					sy = str(posy)
					sz = str(posz)
							
				newcommand = TAG_Compound()

				if command == "Summon":
					commandstr = TAG_String(unicode("summon {} {} {} {} {}".format(unicode(type), unicode(sx), unicode(sy), unicode(sz), unicode(spawndata)).decode("unicode-escape")))
				else:
					if "Item" in ent:
						id = ent["Item"]["id"].value
						damage = str(ent["Item"]["Damage"].value)
						count = str(ent["Item"]["Count"].value)
						if "tag" in ent["Item"]:
							meta = "{"+Spawner2Command(ent["Item"]["tag"])+"}"
					else:
						id = ent["id"].value
						damage = (str(ent["Damage"].value)) if ent["Damage"].value != 0 else ""
						count = str(ent["Count"].value)
						if "tag" in ent:
							meta = "{"+Spawner2Command(ent["tag"])+"}"						
					commandstr = TAG_String(unicode("give {} {} {} {} {}".format(unicode(target), unicode(id), unicode(count), unicode(damage), unicode(meta)).decode("unicode-escape")))				

				if blocktype == "Command Blocks":
					newcommand["id"] = TAG_String("Control")
					newcommand["x"] = TAG_Int(x)
					newcommand["y"] = TAG_Int(y)
					newcommand["z"] = TAG_Int(z)
					level.setBlockAt(x, y, z, 137)
					level.setBlockDataAt(x, y, z, 0)
				else:
					newcommand["id"] = TAG_String("MinecartCommandBlock")
					newcommand["Fire"] = TAG_Short(-1)
					newcommand["OnGround"] = TAG_Byte(0)
					newcommand["FallDistance"] = TAG_Float(0.0)
					newcommand["Rotation"] = TAG_List([TAG_Float(0.0),TAG_Float(0.0)])
					newcommand["Motion"] = TAG_List([TAG_Double(0.0),TAG_Double(0.0),TAG_Double(0.0)])
					newcommand["Command"] = command
					newcommand["CustomName"] = TAG_String("@")
					newcommand["Pos"] = TAG_List([TAG_Double(float(x)+0.5),TAG_Double(float(y)+0.5),TAG_Double(float(z)+0.5)])
					level.setBlockAt(x, y, z, 0)
					level.setBlockDataAt(x, y, z, 0)
				newcommand["Command"] = commandstr
				entsToAdd.append((chunk,newcommand))
				entsToDelete.append((chunk, e))

	for (chunk, entity) in entsToAdd:
		if blocktype == "Command Blocks":
			chunk.TileEntities.append(entity)
		else:
			chunk.Entities.append(entity)
		chunk.dirty = True
				
	for (chunk, entity) in entsToDelete:
		chunk.TileEntities.remove(entity)
		chunk.dirty = True


